#-*- coding:utf-8 -*-

import os
import os.path as Path
from config import Config


class Enhac(object):
    modules_cls = []

    def __init__(self, fname, args=None, debug=False):
        self.debug = debug
        self.fname = Path.abspath(fname)
        self.args = args
        d, f = Path.split(self.fname)
        self.fname_out = Path.join(d, ".enhac_c.%s" % f)

        self.config = Config.lookup(self.fname)
        self.lineno = 0

        self.modules = {}
        self.processors = {}
        for mod_cls in self.modules_cls:
            mod = mod_cls(self)
            self.modules[mod.NAME] = mod
            for proc_name in mod_cls.__dict__:
                if is_valid_name(proc_name):
                    assert proc_name not in self.processors, (
                        "Processor %r (%s) redefined by mod %s" % (proc_name, self.processors[proc_name], mod))

                    self.processors[proc_name] = getattr(mod, proc_name)

    @classmethod
    def register(cls, mod):
        assert hasattr(mod, "ENHAC_MODULE")
        cls.modules_cls.append(mod)

    def __getattr__(self, attr):
        if is_valid_name(attr):
            return self.processors.get(attr, None)
        return object.__getattr__(self, attr)

    def process(self):
        with open(self.fname) as self.fin:
            with open(self.fname_out, "w") as self.fout:
                self._process()

    def _process(self):
        for self.lineno, line in enumerate(self.fin):
            self.lineno += 1
            self.w_line(line)
            if not line.startswith("//:"):
                continue

            directive = line[3:].rstrip("\n")
            if " " not in directive:
                directive += " "
            cmd, arg = directive.split(" ", 1)

            processor = getattr(self, "f_%s" % cmd, None)
            if not processor:
                err = "Unknown command: %r (line %r)" % (cmd, line)
                self.p_warning(err)
                self.w_warning(err)
                continue

            processor(arg)
        self.lineno = "-"

    def w_line(self, line):
        self.fout.write(line.rstrip("\n") + "\n")

    def w_warning(self, msg):
        self.w_line("//! #%s: %s" % (self.lineno, msg))

    def w_close(self):
        self.fout.close()

    def p_warning(self, msg):
        print "WARNING: #%s: %s" % (self.lineno, msg)

    def p_debug(self, msg):
        if self.debug:
            print "DEBUG: #%s: %s" % (self.lineno, msg)

    def p_info(self, msg):
        print "INFO: #%s: %s" % (self.lineno, msg)

    def pw_warning(self, msg):
        self.p_warning(msg)
        self.w_warning(msg)

    def safe_remove(self, fname):
        assert Path.basename(fname).startswith(".enhac_")
        try:
            os.unlink(fname)
        except:
            pass


def is_valid_name(attr):
    return attr.startswith("f_") or attr.startswith("do_")
